'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleAtRule = handleAtRule;

var _cssAtRules = require('../../constants/cssAtRules');

var _cssBrowserHacks = require('../../constants/cssBrowserHacks');

var _reCssMediaQueryFeature = require('../../constants/reCssMediaQueryFeature');

var _reCssMediaQueryType = require('../../constants/reCssMediaQueryType');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _reUrlFunctionWithArg = require('../../constants/reUrlFunctionWithArg');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

var _unique = require('../../converters/unique');

var _handleVendorPrefix = require('../handleVendorPrefix');

var reOnlyKeyword = /\bonly\b/g;
var cssMediaBrowserHacks = _cssBrowserHacks.cssBrowserHacks.filter(function (hack) {
	return hack.type === 'media';
});

function handleMediaQueryParams(params, report, options) {
	report.mediaQueries.total++;

	var cleanedParams = (0, _removeExtraSpaces.removeExtraSpaces)(params);

	if (reOnlyKeyword.test(cleanedParams)) {
		report.mediaQueries.onlyKeywords += cleanedParams.match(reOnlyKeyword).length;
	}

	if (_reCssMediaQueryType.reCssMediaQueryType.test(cleanedParams)) {
		cleanedParams.match(_reCssMediaQueryType.reCssMediaQueryType).forEach(function (type) {
			return (0, _countUsage.countUsage)(type, report.mediaQueries.types);
		});
	}

	if (_reCssMediaQueryFeature.reCssMediaQueryFeature.test(cleanedParams)) {
		cleanedParams.match(_reCssMediaQueryFeature.reCssMediaQueryFeature).forEach(function (feature) {
			if (/^(min|max)--moz-/.test(feature)) {
				report.mediaQueries.vendorPrefixedFeatures++;
				(0, _handleVendorPrefix.handleVendorPrefix)('-moz-', report);
			} else if (_rePrefixedString.rePrefixedString.test(feature)) {
				report.mediaQueries.vendorPrefixedFeatures++;
				(0, _handleVendorPrefix.handleVendorPrefix)(feature, report);
			}

			(0, _countUsage.countUsage)(feature, report.mediaQueries.features);
		});
	}

	(0, _countUsage.countUsage)(cleanedParams, report.mediaQueries.usage);

	if (options.collectBrowserHacksData) {
		cssMediaBrowserHacks.forEach(function (hack) {
			if (hack.regex.test(cleanedParams)) {
				cleanedParams.match(hack.regex).forEach(function (match) {
					report.browserHacks.total++;
					(0, _countUsage.countUsage)(match, report.browserHacks.usage.media);
				});
			}
		});
	}
}

var reCssFileString = /['"]?.+\.css['"]?/g;

function handleImportAtRule(atRule, report, options) {
	var cleanedParams = atRule.params;

	if (_reUrlFunctionWithArg.reUrlFunctionWithArg.test(cleanedParams)) {
		cleanedParams = cleanedParams.replace(_reUrlFunctionWithArg.reUrlFunctionWithArg, '');
	}

	if (reCssFileString.test(cleanedParams)) {
		cleanedParams = cleanedParams.replace(reCssFileString, '');
	}

	if (cleanedParams.trim().length !== 0) {
		handleMediaQueryParams(cleanedParams, report, options);
	}
}

var cssSupportsBrowserHacks = _cssBrowserHacks.cssBrowserHacks.filter(function (hack) {
	return hack.type === 'supports';
});

function handleSupportsAtRule(atRule, report, options) {
	if (options.collectBrowserHacksData) {
		cssSupportsBrowserHacks.forEach(function (hack) {
			if (hack.regex.test(atRule.params)) {
				atRule.params.match(hack.regex).forEach(function (match) {
					report.browserHacks.total++;
					(0, _countUsage.countUsage)(match, report.browserHacks.usage.supports);
				});
			}
		});
	}
}

function handleMediaAtRule(atRule, report, options) {
	handleMediaQueryParams(atRule.params, report, options);
}

var reNumberPercentage = /^(?:[0-9]+\.)?[0-9]+%$/;

function handleKeyframesAtRule(atRule, report, options) {
	var steps = [];

	atRule.walkRules(function (rule) {
		rule.selector.replace(/,\s+/g, ',').split(',').map(function (selector) {
			return selector.trim();
		}).forEach(function (part) {
			if (part === 'from') {
				steps.push(0);
			}

			if (reNumberPercentage.test(part)) {
				steps.push(Number(part.slice(0, -1)));
			}

			if (part === 'to') {
				steps.push(100);
			}
		});

		rule.walkDecls(function (decl) {
			if (report.keyframes.animatableProperties.includes(decl.prop) === false) {
				report.keyframes.animatableProperties.push(decl.prop);
			}
		});
	});

	var stepsUnique = (0, _unique.unique)(steps);

	var stepsChain = stepsUnique.sort(function (a, b) {
		return a - b;
	}).map(function (step) {
		return step + '%';
	}).join(' -> ');

	(0, _countUsage.countUsage)(stepsChain, report.keyframes.stepsChains);

	if (report.keyframes.longestStepsChainLength < stepsUnique.length) {
		report.keyframes.longestStepsChainLength = stepsUnique.length;
		report.keyframes.longestStepsChain = stepsChain;
	}

	if (report.keyframes.shortestStepsChainLength > stepsUnique.length) {
		report.keyframes.shortestStepsChainLength = stepsUnique.length;
		report.keyframes.shortestStepsChain = stepsChain;
	}
}

function handleAtRule(atRule, report, options) {
	report.atRules.total++;

	if (Array.isArray(atRule.nodes) && atRule.nodes.length === 0) {
		report.atRules.empty++;
	}

	(0, _countUsage.countUsage)(atRule.name, report.atRules.usage);

	/** Count vendor prefixes in at-rules names */
	if (_rePrefixedString.rePrefixedString.test(atRule.name)) {
		report.atRules.vendorPrefixed++;
		(0, _handleVendorPrefix.handleVendorPrefix)(atRule.name, report);
	}

	/** Count unknown at-rules */
	var unprefixedAtRuleName = atRule.name.replace(_rePrefixedString.rePrefixedString, '');
	if (_cssAtRules.cssAtRules.includes(unprefixedAtRuleName) === false) {
		(0, _countUsage.countUsage)(atRule.name, report.atRules.unknown);
	}

	if (atRule.name === 'import') {
		handleImportAtRule(atRule, report, options);
	}

	if (atRule.name === 'supports') {
		handleSupportsAtRule(atRule, report, options);
	}

	if (atRule.name === 'media') {
		handleMediaAtRule(atRule, report, options);
	}

	if (unprefixedAtRuleName === 'keyframes') {
		handleKeyframesAtRule(atRule, report, options);
	}

	atRule.walkDecls(function (decl) {
		(0, _countUsage.countUsage)(atRule.name, report.declarations.inAtRules);
	});
}