'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleColorableProperty = handleColorableProperty;

var _reCssNamedColor = require('../../constants/reCssNamedColor');

var _reCssSystemColor = require('../../constants/reCssSystemColor');

var _reHexColor = require('../../constants/reHexColor');

var _reHexaColor = require('../../constants/reHexaColor');

var _reRgbColor = require('../../constants/reRgbColor');

var _reRgbaColor = require('../../constants/reRgbaColor');

var _reHslColor = require('../../constants/reHslColor');

var _reHslaColor = require('../../constants/reHslaColor');

var _reHwbColor = require('../../constants/reHwbColor');

var _restoreFullHex = require('../../converters/restoreFullHex');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

/**
 * https://www.w3.org/TR/css-color-3/#currentcolor
 * https://www.w3.org/TR/css-color-4/#currentcolor-color
 */
var reCurrentColor = /\bcurrent[cC]olor\b/g;

function countCurrentColors(prop, match, report, options) {
	var reportSection = prop.startsWith('background') ? report.backgroundColors : report.colors;

	match.forEach(function (keyword) {
		reportSection.total++;
		reportSection.currentColorKeyword++;
		(0, _countUsage.countUsage)(keyword, reportSection.usage);

		if (options.collectAllColorsData) {
			report.allColors.total++;
			report.allColors.currentColorKeyword++;
			(0, _countUsage.countUsage)(keyword, report.allColors.usage);
		}
	});
}

var reTransparentColor = /\btransparent\b/g;

function countTransparentColors(prop, match, report, options) {
	var reportSection = prop.startsWith('background') ? report.backgroundColors : report.colors;

	match.forEach(function (keyword) {
		reportSection.total++;
		reportSection.transparentKeyword++;
		(0, _countUsage.countUsage)(keyword, reportSection.usage);

		if (options.collectAllColorsData) {
			report.allColors.total++;
			report.allColors.transparentKeyword++;
			(0, _countUsage.countUsage)(keyword, report.allColors.usage);
		}
	});
}

function countNamedColors(prop, match, report, options) {
	var reportSection = prop.startsWith('background') ? report.backgroundColors : report.colors;

	match.forEach(function (color) {
		reportSection.total++;

		/** Color name declaration is not case sensitive */
		var lowerCasedColor = color.toLowerCase();
		(0, _countUsage.countUsage)(lowerCasedColor, reportSection.usage);
		(0, _countUsage.countUsage)(lowerCasedColor, reportSection.named);

		if (options.collectAllColorsData) {
			report.allColors.total++;
			(0, _countUsage.countUsage)(lowerCasedColor, report.allColors.usage);
			(0, _countUsage.countUsage)(lowerCasedColor, report.allColors.named);
		}
	});
}

function countColorModels(prop, match, report, model, options) {
	var reportSection = prop.startsWith('background') ? report.backgroundColors : report.colors;

	match.forEach(function (color) {
		var normalizedColor = void 0;

		if (color.startsWith('#')) {
			normalizedColor = color.length === 4 || color.length === 5 ? (0, _restoreFullHex.restoreFullHex)(color).toLowerCase() : color.toLowerCase();
		} else {
			normalizedColor = (0, _removeExtraSpaces.removeExtraSpaces)(color);
		}

		reportSection.total++;
		(0, _countUsage.countUsage)(normalizedColor, reportSection.usage);
		(0, _countUsage.countUsage)(model, reportSection.models);

		if (options.collectAllColorsData) {
			report.allColors.total++;
			(0, _countUsage.countUsage)(normalizedColor, report.allColors.usage);
			(0, _countUsage.countUsage)(model, report.allColors.models);
		}
	});
}

function countSystemColors(prop, match, report, options) {
	var reportSection = prop.startsWith('background') ? report.backgroundColors : report.colors;

	match.forEach(function (color) {
		reportSection.total++;
		(0, _countUsage.countUsage)(color, reportSection.usage);
		(0, _countUsage.countUsage)(color, reportSection.system);

		if (options.collectAllColorsData) {
			report.allColors.total++;
			(0, _countUsage.countUsage)(color, report.allColors.usage);
			(0, _countUsage.countUsage)(color, report.allColors.system);
		}
	});
}

function handleColorableProperty(decl, report, options) {
	var prop = decl.prop;
	var propValue = decl.value;

	if (reCurrentColor.test(propValue)) {
		countCurrentColors(prop, propValue.match(reCurrentColor), report, options);
	}

	if (reTransparentColor.test(propValue)) {
		countTransparentColors(prop, propValue.match(reTransparentColor), report, options);
	}

	if (_reCssNamedColor.reCssNamedColor.test(propValue)) {
		countNamedColors(prop, propValue.match(_reCssNamedColor.reCssNamedColor), report, options);
	}

	if (_reHexColor.reHexColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHexColor.reHexColor), report, 'hex', options);
	}

	if (_reHexaColor.reHexaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHexaColor.reHexaColor), report, 'hexa', options);
	}

	if (_reRgbColor.reRgbColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reRgbColor.reRgbColor), report, 'rgb', options);
	}

	if (_reRgbaColor.reRgbaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reRgbaColor.reRgbaColor), report, 'rgba', options);
	}

	if (_reHslColor.reHslColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHslColor.reHslColor), report, 'hsl', options);
	}

	if (_reHslaColor.reHslaColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHslaColor.reHslaColor), report, 'hsla', options);
	}

	if (_reHwbColor.reHwbColor.test(propValue)) {
		countColorModels(prop, propValue.match(_reHwbColor.reHwbColor), report, 'hwb', options);
	}

	if (_reCssSystemColor.reCssSystemColor.test(propValue)) {
		countSystemColors(prop, propValue.match(_reCssSystemColor.reCssSystemColor), report, options);
	}
}