'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleDeclaration = handleDeclaration;

var _cssPropertyParser = require('css-property-parser');

var _cssEngineTriggerProperties = require('../../constants/cssEngineTriggerProperties');

var _cssColorableProperties = require('../../constants/cssColorableProperties');

var _cssExplicitDefaultingKeywords = require('../../constants/cssExplicitDefaultingKeywords');

var _reCssExplicitDefaultingKeyword = require('../../constants/reCssExplicitDefaultingKeyword');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _handleColorableProperty = require('../handleColorableProperty');

var _handleFontProperties = require('../handleFontProperties');

var _handleTransitionsAndAnimations = require('../handleTransitionsAndAnimations');

var _handleFunctions = require('../handleFunctions');

var _handleUnits = require('../handleUnits');

var _handleVariables = require('../handleVariables');

var _handlePerformanceHackProperties = require('../handlePerformanceHackProperties');

var _handleVendorPrefix = require('../handleVendorPrefix');

var _countUsage = require('../../calculators/countUsage');

var _removeExtraSpaces = require('../../converters/removeExtraSpaces');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function isFontDeclarationInAtRule(decl) {
	return decl.parent.type === 'atrule' && decl.parent.name === 'font-face' && decl.prop.startsWith('font-') === true;
}

function countEngineTriggerProperties(prop, report) {
	if (Object.keys(_cssEngineTriggerProperties.cssEngineTriggerProperties).includes(prop)) {
		var engineTriggers = _cssEngineTriggerProperties.cssEngineTriggerProperties[prop];

		Object.keys(engineTriggers).forEach(function (engine) {
			if (engineTriggers[engine].layout === true) {
				report.properties.engineTriggers.layout[engine]++;
			}
			if (engineTriggers[engine].paint === true) {
				report.properties.engineTriggers.paint[engine]++;
			}
			if (engineTriggers[engine].composite === true) {
				report.properties.engineTriggers.composite[engine]++;
			}
		});
	}
}

function handleDeclaration(decl, report, options) {
	report.declarations.total++;

	var normalizedDecl = (0, _removeExtraSpaces.removeExtraSpaces)(decl.toString());

	if (report.declarations.list.includes(normalizedDecl) === false) {
		report.declarations.list.push(normalizedDecl);
	}

	if (decl.important) {
		report.declarations.important++;
	}

	var prop = decl.prop;
	var propValue = decl.value;

	/** Count properties excluding variables */
	if (prop.startsWith('--') === false) {
		report.properties.total++;
		(0, _countUsage.countUsage)(prop, report.properties.usage);
	}

	/** Count properties with vendor prefixes */
	if (_rePrefixedString.rePrefixedString.test(prop)) {
		report.properties.vendorPrefixed++;
		(0, _handleVendorPrefix.handleVendorPrefix)(prop, report);

		/** Count property shorthands */
		// https://github.com/mahirshah/css-property-parser/issues/31
		var unprefixedProp = prop.replace(_rePrefixedString.rePrefixedString, '');
		if ((0, _cssPropertyParser.isShorthandProperty)(unprefixedProp)) {
			report.properties.shorthands++;
		}
	} else if ((0, _cssPropertyParser.isShorthandProperty)(prop)) {
		report.properties.shorthands++;
	}

	if ((options.collectColorsData || options.collectBackgroundColorsData || options.collectAllColorsData) && _cssColorableProperties.cssColorableProperties.includes(prop)) {
		(0, _handleColorableProperty.handleColorableProperty)(decl, report, options);
	}

	if (options.collectEngineTriggerProperties) {
		countEngineTriggerProperties(prop, report);
	}

	/**
  * Count property values resets via `all` property
  * https://drafts.csswg.org/css-cascade-4/#all-shorthand
  */
	if (prop === 'all') {
		report.properties.resetsViaAll++;
	}

	/**
  * Count anonymous replaced elements
  * https://developer.mozilla.org/en-US/docs/Web/CSS/Replaced_element
  */
	if (prop === 'content') {
		report.properties.anonymousReplacedElements++;
	}

	/** Count displays */
	if (prop === 'display') {
		report.displays.total++;
		(0, _countUsage.countUsage)(propValue, report.displays.usage);
	}

	/** Count positions */
	if (prop === 'position') {
		report.positions.total++;
		(0, _countUsage.countUsage)(propValue, report.positions.usage);
	}

	/** Count floats */
	if (prop === 'float') {
		report.floats.total++;
		(0, _countUsage.countUsage)(propValue, report.floats.usage);
	}

	/** Count border radiuses */
	if (prop.startsWith('--') === false && prop.includes('border-') && prop.endsWith('-radius')) {
		report.borderRadiuses.total++;
		(0, _countUsage.countUsage)(propValue, report.borderRadiuses.usage);
	}

	/** Count letter spacings */
	if (prop === 'letter-spacing') {
		report.letterSpacings.total++;
		(0, _countUsage.countUsage)(propValue, report.letterSpacings.usage);
	}

	/** Count z-indices */
	if (prop === 'z-index') {
		report.zIndices.total++;
		(0, _countUsage.countUsage)(propValue, report.zIndices.usage);

		/** Count invalid z-indices */
		if (Number.isInteger(Number(propValue)) === false && ['auto'].concat(_toConsumableArray(_cssExplicitDefaultingKeywords.cssExplicitDefaultingKeywords)).includes(propValue) === false) {
			(0, _countUsage.countUsage)(propValue, report.zIndices.invalid);
		}
	}

	if (options.collectFontsData && isFontDeclarationInAtRule(decl) === false && ['font', 'font-size', 'line-height', 'font-family'].includes(prop)) {
		(0, _handleFontProperties.handleFontProperties)(decl, report);
	}

	if (options.collectFunctionsData) {
		(0, _handleFunctions.handleFunctions)(decl, report, options);
	}

	if (options.collectTransitionsAndAnimationsData && (prop.includes('transition') || prop.includes('animation'))) {
		(0, _handleTransitionsAndAnimations.handleTransitionsAndAnimations)(decl, report);
	}

	if (options.collectVariablesData) {
		(0, _handleVariables.handleVariables)(decl, report);
	}

	if (options.collectUnitsData) {
		(0, _handleUnits.handleUnits)(decl, report);
	}

	if (options.collectPerformanceHacksData) {
		(0, _handlePerformanceHackProperties.handlePerformanceHackProperties)(decl, report);
	}

	if (_reCssExplicitDefaultingKeyword.reCssExplicitDefaultingKeyword.test(propValue)) {
		propValue.match(_reCssExplicitDefaultingKeyword.reCssExplicitDefaultingKeyword).forEach(function (match) {
			report.properties.explicitDefaultingKeywords.total++;
			(0, _countUsage.countUsage)(match, report.properties.explicitDefaultingKeywords);
		});
	}
}