'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.handleTransitionsAndAnimations = handleTransitionsAndAnimations;

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _cssPropertyParser = require('css-property-parser');

var _cssNamedTimingFunctions = require('../../constants/cssNamedTimingFunctions');

var _cssExplicitDefaultingKeywords = require('../../constants/cssExplicitDefaultingKeywords');

var _rePrefixedString = require('../../constants/rePrefixedString');

var _countUsage = require('../../calculators/countUsage');

var _isNumber = require('../../predicates/isNumber');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var reInfiniteKeyword = /\binfinite\b/g;

function countInfiniteAnimations(propValue, report) {
	if (reInfiniteKeyword.test(propValue)) {
		report.animations.infinite += propValue.match(reInfiniteKeyword).length;
	}
}

function countNamedTimingFunctions(decl, report) {
	var reportSection = decl.prop.includes('animation') ? report.animations : report.transitions;

	_postcss2.default.list.comma(decl.value).forEach(function (func) {
		if (_cssNamedTimingFunctions.cssNamedTimingFunctions.includes(func) || _cssExplicitDefaultingKeywords.cssExplicitDefaultingKeywords.includes(func)) {
			(0, _countUsage.countUsage)(func, reportSection.timingFunctions);
		} else {
			(0, _countUsage.countUsage)(func, reportSection.invalidTimingFunctions);
		}
	});
}

var reValueWithDurationUnit = /^-?(\.?[0-9]+)(\.[0-9]+)?(ms|s)$/g;

function countDurations(decl, report) {
	var reportSection = decl.prop.includes('animation') ? report.animations : report.transitions;

	_postcss2.default.list.comma(decl.value).forEach(function (duration) {
		if (duration.match(reValueWithDurationUnit) !== null) {
			var parsedDurationInSeconds = duration.endsWith('ms') ? parseFloat(duration) / 1000 : parseFloat(duration);

			if ((0, _isNumber.isNumber)(parsedDurationInSeconds) && reportSection.longestDuration < parsedDurationInSeconds) {
				reportSection.longestDuration = parsedDurationInSeconds;
			}

			if ((0, _isNumber.isNumber)(parsedDurationInSeconds) && reportSection.shortestDuration > parsedDurationInSeconds) {
				reportSection.shortestDuration = parsedDurationInSeconds;
			}
		}
	});
}

function countDelays(decl, report) {
	var reportSection = decl.prop.includes('animation') ? report.animations : report.transitions;

	_postcss2.default.list.comma(decl.value).forEach(function (delay) {
		if (delay.match(reValueWithDurationUnit) !== null) {
			var parsedDelayInSeconds = delay.endsWith('ms') ? parseFloat(delay) / 1000 : parseFloat(delay);

			if ((0, _isNumber.isNumber)(parsedDelayInSeconds) && reportSection.longestDelay < parsedDelayInSeconds) {
				reportSection.longestDelay = parsedDelayInSeconds;
			}

			if ((0, _isNumber.isNumber)(parsedDelayInSeconds) && reportSection.shortestDelay > parsedDelayInSeconds) {
				reportSection.shortestDelay = parsedDelayInSeconds;
			}
		}
	});
}

function countTransitionablePropeties(propValue, report) {
	_postcss2.default.list.comma(propValue).forEach(function (prop) {
		return (0, _countUsage.countUsage)(prop, report.transitions.properties);
	});
}

function handleAnimationLonghand(longhand, report) {
	if (typeof longhand['animation-iteration-count'] === 'string') {
		countInfiniteAnimations(longhand['animation-iteration-count'], report);
	}

	if (typeof longhand['animation-timing-function'] === 'string') {
		var newDecl = {
			prop: 'animation-timing-function',
			value: longhand['animation-timing-function']
		};

		countNamedTimingFunctions(newDecl, report);
	}

	if (typeof longhand['animation-duration'] === 'string') {
		var _newDecl = {
			prop: 'animation-duration',
			value: longhand['animation-duration']
		};

		countDurations(_newDecl, report);
	}

	if (typeof longhand['animation-delay'] === 'string') {
		var _newDecl2 = {
			prop: 'animation-delay',
			value: longhand['animation-delay']
		};

		countDelays(_newDecl2, report);
	}
}

function handleTransitionLonghand(longhand, report) {
	if (typeof longhand['transition-timing-function'] === 'string') {
		var newDecl = {
			prop: 'transition-timing-function',
			value: longhand['transition-timing-function']
		};

		countNamedTimingFunctions(newDecl, report);
	}

	if (typeof longhand['transition-duration'] === 'string') {
		var _newDecl3 = {
			prop: 'transition-duration',
			value: longhand['transition-duration']
		};

		countDurations(_newDecl3, report);
	}

	if (typeof longhand['transition-delay'] === 'string') {
		var _newDecl4 = {
			prop: 'transition-delay',
			value: longhand['transition-delay']
		};

		countDelays(_newDecl4, report);
	}

	if (typeof longhand['transition-property'] === 'string') {
		countTransitionablePropeties(longhand['transition-property'], report);
	}
}

var reCubicBezierInexact = /cubic-bezier\(.+?\)/g;

function handleTransitionsAndAnimations(decl, report) {
	var prop = decl.prop;
	var propValue = decl.value;

	if ((0, _cssPropertyParser.isShorthandProperty)(prop)) {
		var safePropValue = propValue;

		// https://github.com/mahirshah/css-property-parser/issues/27
		if (_rePrefixedString.rePrefixedString.test(safePropValue)) {
			safePropValue = safePropValue.replace(_rePrefixedString.rePrefixedString, '');
		}

		// https://github.com/mahirshah/css-property-parser/issues/26
		if (reCubicBezierInexact.test(safePropValue)) {
			safePropValue = safePropValue.replace(reCubicBezierInexact, '');
		}

		try {
			var longhand = (0, _cssPropertyParser.expandShorthandProperty)(prop, safePropValue);

			if (prop.includes('animation')) {
				handleAnimationLonghand(longhand, report);
			}

			if (prop.includes('transition')) {
				handleTransitionLonghand(longhand, report);
			}
		} catch (err) {
			/* eslint-disable-next-line no-console */
			console.log('\'css-property-parser\' module error\n' + err);
		}
	}

	if (prop === 'animation-iteration-count') {
		countInfiniteAnimations(propValue, report);
	}

	if (prop === 'animation-timing-function' || prop === 'transition-timing-function') {
		countNamedTimingFunctions(decl, report);
	}

	if (prop === 'animation-duration' || prop === 'transition-duration') {
		countDurations(decl, report);
	}

	if (prop === 'animation-delay' || prop === 'transition-delay') {
		countDelays(decl, report);
	}

	if (prop === 'transition-property') {
		countTransitionablePropeties(propValue, report);
	}
}